@@ -32,6 +32,7 @@
 (require 'cl-lib)
 (require 'subr-x)
 (require 'llama)
+(require 'el-job-ng)
 (require 'org-mem)
 (require 'org-mem-parser)
 
@@ -64,14 +65,18 @@ In that case, there may be nothing wrong with the known name."
           (cached-true (gethash file org-mem--wild-filename<>truename)))
       (when (and cached-true (not (file-symlink-p file)))
         (push cached-true bad))
-      (org-mem-updater--forget-links-from-files bad) ;; REVIEW
       (org-mem-updater--forget-file-contents bad)
       (org-mem--invalidate-file-names bad)
+      (org-mem--rebuild-specially-indexed-tables)
       (mapc #'clrhash (hash-table-values org-mem--key<>subtable)))))
 
 (defun org-mem-updater--handle-save (&optional file)
   "Arrange to scan entries and links in FILE, or current buffer file."
-  (unless file (setq file buffer-file-name))
+  (unless file
+    (if buffer-file-name
+        (setq file buffer-file-name)
+      (error "org-mem-updater--handle-save: Called with nil argument in %S"
+             (current-buffer))))
   (when (and (seq-find (##string-suffix-p % file) org-mem-suffixes)
              (not (backup-file-name-p file)))
     (org-mem-updater--scan-targeted file)))
@@ -84,22 +89,25 @@ In that case, there may be nothing wrong with the known name."
                      (seq-uniq)
                      (seq-filter (##cl-loop for xclude in org-mem-exclude
                                             never (string-search xclude %))))))
-    (el-job-launch :id 'org-mem-updater
-                   :inject-vars (append (org-mem--mk-work-vars) org-mem-inject-vars)
-                   :load-features (append '(org-mem-parser) org-mem-load-features)
+    (el-job-ng-run :inject-vars (append (org-mem--mk-work-vars)
+                                        (el-job-ng-vars org-mem-inject-vars))
+                   :require (append '(org-mem-parser) org-mem-load-features)
                    :inputs truenames
                    :funcall-per-input #'org-mem-parser--parse-file
                    :callback #'org-mem-updater--finalize-targeted-scan)))
 
-(defun org-mem-updater--finalize-targeted-scan (parse-results _job)
+(defun org-mem-updater--finalize-targeted-scan (parse-results)
   "Handle PARSE-RESULTS from `org-mem-updater--scan-targeted'."
+  ;; Transitional #33
+  (let* ((ng-style-results parse-results)
+         (merged (pop ng-style-results)))
+    (while ng-style-results
+      (setq merged (org-mem--zip (pop ng-style-results) merged)))
+    (setq parse-results merged))
   (run-hook-with-args 'org-mem-pre-targeted-scan-functions parse-results)
   (seq-let (bad-paths file-data entries links problems) parse-results
-    (let ((stale-files (append bad-paths (mapcar #'car file-data))))
-      (org-mem-updater--forget-links-from-files stale-files)
-      (org-mem-updater--forget-file-contents stale-files))
-    (when bad-paths
-      (org-mem--invalidate-file-names bad-paths))
+    (org-mem-updater--forget-file-contents (append bad-paths (mapcar #'car file-data)))
+    (org-mem--invalidate-file-names bad-paths)
     (mapc #'clrhash (hash-table-values org-mem--key<>subtable))
     (with-current-buffer
         (setq org-mem-scratch (get-buffer-create " *org-mem-scratch*" t))
@@ -107,15 +115,13 @@ In that case, there may be nothing wrong with the known name."
         (puthash (car datum) datum org-mem--truename<>metadata)
         (run-hook-with-args 'org-mem-record-file-functions datum))
       (dolist (entry entries)
-        (org-mem--record-entry entry))
+        (org-mem--record-entry entry)
+        (run-hook-with-args 'org-mem-record-entry-functions entry))
       (dolist (link links)
-        (org-mem--record-link link)
-        (unless (gethash (org-mem-link-target link)
-                         org-mem-updater--id-or-ref-target<>old-links)
-          (when (or (org-mem-roam-reflink-p link)
-                    (org-mem-id-link-p link))
-            (push (org-mem-link-target link)
-                  org-mem-updater--new-id-or-ref-targets)))))
+        (push link (gethash (org-mem-link--internal-entry-id link)
+                            org-mem--internal-entry-id<>links))
+        (run-hook-with-args 'org-mem-record-link-functions link)))
+    (org-mem--rebuild-specially-indexed-tables)
     (dolist (prob problems)
       (push prob org-mem--problems))
     (run-hook-with-args 'org-mem-post-targeted-scan-functions parse-results)
@@ -127,8 +133,8 @@ In that case, there may be nothing wrong with the known name."
 
 (defun org-mem-updater--forget-file-contents (files)
   "Delete from tables, most info relating to FILES and their contents.
-You should also run `org-mem--invalidate-file-names',
-and potentially `org-mem-updater--forget-links-from-files'."
+You should also run `org-mem--invalidate-file-names'
+and `org-mem--rebuild-specially-indexed-tables'."
   (setq files (ensure-list files))
   (when files
     (with-current-buffer
@@ -137,55 +143,12 @@ and potentially `org-mem-updater--forget-links-from-files'."
         (dolist (entry (gethash file org-mem--truename<>entries))
           (remhash (org-mem-entry-id entry) org-mem--id<>entry)
           (remhash (org-mem-entry-title-maybe entry) org-mem--title<>id)
+          (remhash (org-mem-entry--internal-id entry) org-mem--internal-entry-id<>links)
           (run-hook-with-args 'org-mem-forget-entry-functions entry))
         (remhash file org-mem--truename<>entries)
         (remhash file org-mem--truename<>metadata)
         (run-hook-with-args 'org-mem-forget-file-functions file)))))
 
-;; For downstream use
-(defvar org-mem-updater--id-or-ref-target<>old-links (make-hash-table :test 'equal)
-  "Previous state of `org-mem--target<>links' for some targets.
-Only includes targets that are or were IDs or roam-refs, so that the
-values can be considered \"forward links\" qualified for backlinks under
-some paradigms.
-Values may be in different order.")
-
-(defvar org-mem-updater--new-id-or-ref-targets nil
-  "IDs and ROAM_REFS that had no links targeting them but now do.
-Even if the ID or ref is old, it would previously have had no presence
-in table `org-mem--target<>links'.")
-
-;; FIXME: It seems to remove too many, or they don't all get re-added or something
-(defun org-mem-updater--forget-links-from-files (stale-files)
-  "Remove from tables, all links in STALE-FILES.
-Also clear `org-mem-updater--id-or-ref-target<>old-links' and stash into it the
-relevant rows from `org-mem--target<>links' prior to updating those rows
-in the latter table."
-  (clrhash org-mem-updater--id-or-ref-target<>old-links)
-  (let ((stale-eids (mapcar #'org-mem-entry--internal-id
-                            (org-mem-entries-in-files stale-files)))
-        targets-to-update)
-    (dolist (eid stale-eids)
-      (dolist (link (gethash eid org-mem--internal-entry-id<>links))
-        (cl-pushnew (org-mem-link-target link) targets-to-update
-                    :test #'equal))
-      (remhash eid org-mem--internal-entry-id<>links))
-    (dolist (tgt targets-to-update)
-      (cl-loop
-       for link in (gethash tgt org-mem--target<>links)
-       if (memq (org-mem-link--internal-entry-id link) stale-eids)
-       collect link into stale-links
-       else collect link into reduced-link-set
-       finally do
-       (puthash tgt reduced-link-set org-mem--target<>links)
-       (dolist (link stale-links)
-         (run-hook-with-args 'org-mem-forget-link-functions link))
-       (dolist (link (nconc stale-links reduced-link-set))
-         ;; TODO: Don't waste perf checking every link, check tgt once
-         (when (or (org-mem-roam-reflink-p link)
-                   (org-mem-id-link-p link))
-           (push link (gethash tgt org-mem-updater--id-or-ref-target<>old-links))))))))
-
 
 ;;; Instant placeholders
 
@@ -242,7 +205,10 @@ No support for citations."
                  (cl-notany (##string-search % truename)
                             org-mem-exclude))
         (org-mem-updater-ensure-buffer-file-known)
-        (org-mem--record-link (org-mem-updater-mk-link-atpt))))))
+        (let ((link (org-mem-updater-mk-link-atpt)))
+          (push link (gethash (org-mem-link--internal-entry-id link)
+                              org-mem--internal-entry-id<>links))
+          (run-hook-with-args 'org-mem-record-link-functions link))))))
 
 (defun org-mem-updater-ensure-id-node-at-point-known ()
   "Record ID-node at point.
@@ -280,13 +246,15 @@ No support for citations."
                 truename
                 (point)
                 type
-                target
+                target ;; HACK: includes supplement
                 (and desc-beg (buffer-substring-no-properties desc-beg desc-end))
                 nil
                 (org-entry-get-with-inheritance "ID")
-                (org-mem-parser--mk-id
-                 truename (if (org-before-first-heading-p) 0
-                            (org-entry-beginning-position)))))
+                nil ;; HACK: supplement field is nil
+                (+ (org-mem-parser--hash truename)
+                   (if (org-before-first-heading-p)
+                       0
+                     (org-entry-beginning-position)))))
     (error "No link at point %d in %s" (point) (current-buffer))))
 
 (defun org-mem-updater-mk-entry-atpt ()
@@ -340,7 +308,7 @@ Some fields are incomplete or left at nil."
             (org-mem-updater--tags-at-point-inherited-only)
             (org-get-tags nil t)
             (when heading (org-get-todo-state))
-            (org-mem-parser--mk-id truename (if heading pos 0)))))
+            (+ (org-mem-parser--hash truename) (if heading pos 0)))))
 
 (defun org-mem-updater--tags-at-point-inherited-only ()
   "Like `org-get-tags', but get only the inherited tags."
@@ -393,8 +361,8 @@ Override this if you prefer different timer delays, or no timer."
     (advice-remove 'delete-file                    #'org-mem-updater--handle-delete)
     (cancel-timer org-mem-updater--timer)))
 
-(org-mem--def-whiny-alias 'org-mem-updater-ensure-entry-at-point-known   #'org-mem-updater-ensure-id-node-at-point-known "2025-05-21" "November")
-(org-mem--def-whiny-alias 'org-mem-updater--activate-timer   #'org-mem-updater--adjust-timer "2025-05-24" "November")
+(defvar org-mem-updater--id-or-ref-target<>old-links :obsolete)
+(defvar org-mem-updater--new-id-or-ref-targets :obsolete)
 
 (provide 'org-mem-updater)
 
